﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Shapes;

namespace CSSAMPLE
{
    /// <summary>
    /// CSWindow.xaml の相互作用ロジック
    /// </summary>
    public partial class CSWindow : Window
    {
        public CSWindow()
        {
            InitializeComponent();
        }


        private TurtleLib.TurtleCommandSynchroniser _tcs;


        /// <summary>
        /// Connectボタンクリック時のイベントハンドラです．
        /// TCP通信とコマンドの同期を提供するTurtleCommandSynchroniserクラスを初期化してホストに接続します．
        /// またConnectボタンのTextを変更します．
        /// </summary>
        private void btnConnect_Click(object sender, RoutedEventArgs e)
        {
			if(btnConnect.Content as string == "Connect")
			{
				int port;
				if(!int.TryParse(tbPort.Text, out port))
					return;
			
				_tcs = new TurtleLib.TurtleCommandSynchroniser();
                _tcs.SocketMessageRaised += new System.Action<String>(this.tsock_SocketStateChanged);
                _tcs.SocketConnected += new System.Action(this.tsock_SocketConnected);
				_tcs.SocketClosed        += new System.Action(this.tsock_SocketClosed);
				_tcs.ResponseArrived     += new System.Action<String, String>(this.tsc_ResponseArrived);

				_tcs.Connect(tbHostName.Text, port);
				
				btnConnect.Content = "Connecting...";
			}
            else if (btnConnect.Content as string == "Close" || btnConnect.Content as string == "Connecting...")
			{
				if(_tcs!=null)
				{
					_tcs.Close();
					_tcs = null;
				}
			}
        }


        /// <summary>
        /// プロダクトコード取得ボタンクリック時のイベントハンドラです．
        /// pcodeコマンドを送信します．
        /// </summary>
        private void btnProductCode_Click(object sender, RoutedEventArgs e)
        {
            if (btnConnect.Content as string != "Close")
                return;

            if (_tcs != null)
                _tcs.Send("pcode\n");
        }


        /// <summary>
        /// 機器情報取得ボタンクリック時のイベントハンドラです．
        /// infoコマンドを送信します．
        /// </summary>
        private void btnInfo_Click(object sender, RoutedEventArgs e)
        {
            if (btnConnect.Content as string != "Close")
                return;

            if (_tcs != null)
                _tcs.Send("info\n");
        }


        /// <summary>
        /// リードボタンクリック時のイベントハンドラです．
        /// readコマンドを送信します．
        /// </summary>
        private void btnRead_Click(object sender, RoutedEventArgs e)
        {
            if (btnConnect.Content as string != "Close")
                return;

            if (_tcs != null)
                _tcs.Send("read\n");
        }


        /// <summary>
        /// ライトボタンクリック時のイベントハンドラです．
        /// writeコマンドを送信します．
        /// </summary>
        private void btnWrite_Click(object sender, RoutedEventArgs e)
        {
            if (btnConnect.Content as string != "Close")
                return;

            if (_tcs != null)
                _tcs.Send("write " + tbWrite.Text + "\n");
        }

        /// <summary>
        /// このウィンドウを閉じるイベントハンドラです．もしTurtleCommandSynchroniserのインスタンスが存在するなら後始末をします．
        /// </summary>
        private void Window_Unloaded(object sender, RoutedEventArgs e)
        {
            if (_tcs != null)
            {
                _tcs.Close();
                _tcs = null;
            }     
        }


        /// <summary>
        /// TurtleCommandSynchroniserクラスのSocketConnectedイベント発生時の処理です．
        /// ConnectボタンのTextを変更します．
        /// </summary>
        private void tsock_SocketConnected()
        {
            btnConnect.Content = "Close";
        }


        /// <summary>
        /// TurtleCommandSynchroniserクラスのSocketClosedイベント発生時の処理です．
        /// ConnectボタンのTextを変更します．
        /// </summary>
        private void tsock_SocketClosed()
        {
            btnConnect.Content = "Connect";
        }



        /// <summary>
        /// TurtleCommandSynchroniserクラスのSocketStateChangedイベント発生時の処理です．
        /// 通知されたstringをテキストボックスに追加します．
        /// </summary>
        private void tsock_SocketStateChanged(String message)
        {
            SocketMessage_Add(message);
        }


        /// <summary>
        /// SocketMessageテキストボックスに文字列を追加します．
        /// </summary>
        private void SocketMessage_Add(String message)
        {
            tbSocketMessage.Text = message + System.Environment.NewLine + tbSocketMessage.Text;
        }



        /// <summary>
        /// TurtleCommandSynchroniserクラスのResponseArrivedイベント発生時の処理です．
        /// 送信したコマンドに応じて応答を処理します．
        /// </summary>
        /// <param name="cmd">送信したコマンド</param>
        /// <param name="res">応答</param>
        private void tsc_ResponseArrived(String cmd, String res)
        {
            if (cmd.StartsWith("pcode"))
            {
                if (res.StartsWith("0001"))
                    labProductCode.Content = "0001: TLAN-INS8/8";
                else if (res.StartsWith("0002"))
                    labProductCode.Content = "0002: TLAN-INS16I";
                else if (res.StartsWith("0003"))
                    labProductCode.Content = "0003: TLAN-INS16O";
                else if (res.StartsWith("0004"))
                    labProductCode.Content = "0004: TLAN-08VMA";
                else if (res.StartsWith("0005"))
                    labProductCode.Content = "0005: TLAN-08VMD";
                else if (res.StartsWith("0006"))
                    labProductCode.Content = "0006: TLAN-RL";
                else
                    labProductCode.Content = "Error or Unknown Device";
            }
            else if (cmd.StartsWith("info"))
            {
                labInfo.Content = res;
            }
            else if (cmd.StartsWith("read"))
            {
                labRead.Content = res;
            }
            else if (cmd.StartsWith("write"))
            {
                labWrite.Content = res;
            }
        }

    }
}
