﻿

Public Class VBWindow

    Private _tcs As TurtleLib.TurtleCommandSynchroniser

    ''' <summary>
    ''' Connectボタンクリック時のイベントハンドラです．
    ''' TCP通信とコマンドの同期を提供するTurtleCommandSynchroniserクラスを初期化してホストに接続します．
    ''' またConnectボタンのTextを変更します．
    ''' </summary>
    Private Sub btnConnect_Click(sender As Object, e As Windows.RoutedEventArgs)

        If CType(btnConnect.Content, String) = "Connect" Then

            Dim port As Integer

            If Not Integer.TryParse(tbPort.Text, port) Then
                Return
            End If

            _tcs = New TurtleLib.TurtleCommandSynchroniser()
            AddHandler _tcs.SocketMessageRaised, AddressOf Me.tsock_SocketStateChanged
            AddHandler _tcs.SocketConnected, AddressOf Me.tsock_SocketConnected
            AddHandler _tcs.SocketClosed, AddressOf Me.tsock_SocketClosed
            AddHandler _tcs.ResponseArrived, AddressOf Me.tsc_ResponseArrived

            _tcs.Connect(tbHostName.Text, port)
            btnConnect.Content = "Connecting..."

        ElseIf CType(btnConnect.Content, String) = "Close" OrElse CType(btnConnect.Content, String) = "Connecting..." Then

            If _tcs IsNot Nothing Then
                _tcs.Close()
                _tcs = Nothing
            End If

        End If

    End Sub


    ''' <summary>
    ''' プロダクトコード取得ボタンクリック時のイベントハンドラです．
    ''' pcodeコマンドを送信します．
    ''' </summary>
    Private Sub btnProductCode_Click(sender As Object, e As Windows.RoutedEventArgs)

        If CType(btnConnect.Content, String) <> "Close" Then
            Return
        End If

        If _tcs IsNot Nothing Then
            _tcs.Send("pcode" & vbLf)
        End If

    End Sub


    ''' <summary>
    ''' 機器情報取得ボタンクリック時のイベントハンドラです．
    ''' infoコマンドを送信します．
    ''' </summary>
    Private Sub btnInfo_Click(sender As Object, e As Windows.RoutedEventArgs)

        If CType(btnConnect.Content, String) <> "Close" Then
            Return
        End If

        If _tcs IsNot Nothing Then
            _tcs.Send("info" & vbLf)
        End If

    End Sub


    ''' <summary>
    ''' リードボタンクリック時のイベントハンドラです．
    ''' readコマンドを送信します．
    ''' </summary>
    Private Sub btnRead_Click(sender As Object, e As Windows.RoutedEventArgs)

        If CType(btnConnect.Content, String) <> "Close" Then
            Return
        End If

        If _tcs IsNot Nothing Then
            _tcs.Send("read" & vbLf)
        End If

    End Sub


    ''' <summary>
    ''' ライトボタンクリック時のイベントハンドラです．
    ''' writeコマンドを送信します．
    ''' </summary>
    Private Sub btnWrite_Click(sender As Object, e As Windows.RoutedEventArgs)

        If CType(btnConnect.Content, String) <> "Close" Then
            Return
        End If

        If _tcs IsNot Nothing Then
            _tcs.Send("write" & tbWrite.Text & vbLf)
        End If

    End Sub


    ''' <summary>
    ''' このウィンドウを閉じるイベントハンドラです．もしTurtleCommandSynchroniserのインスタンスが存在するなら後始末をします．
    ''' </summary>
    Private Sub Window_Unloaded(sender As Object, e As Windows.RoutedEventArgs)

        If _tcs IsNot Nothing Then
            _tcs.Close()
            _tcs = Nothing
        End If

    End Sub


    ''' <summary>
    ''' TurtleCommandSynchroniserクラスのSocketConnectedイベント発生時の処理です．
    ''' ConnectボタンのTextを変更します．
    ''' </summary>
    Private Sub tsock_SocketConnected()
        btnConnect.Content = "Close"
    End Sub


    ''' <summary>
    ''' TurtleCommandSynchroniserクラスのSocketClosedイベント発生時の処理です．
    ''' ConnectボタンのTextを変更します．
    ''' </summary>
    Private Sub tsock_SocketClosed()
        btnConnect.Content = "Connect"
    End Sub



    ''' <summary>
    ''' TurtleCommandSynchroniserクラスのSocketStateChangedイベント発生時の処理です．
    ''' 通知されたstringをテキストボックスに追加します．
    ''' </summary>
    Private Sub tsock_SocketStateChanged(message As String)
        tbSocketMessage.Text = message + System.Environment.NewLine + tbSocketMessage.Text
    End Sub


    ''' <summary>
    ''' TurtleCommandSynchroniserクラスのResponseArrivedイベント発生時の処理です．
    ''' 送信したコマンドに応じて応答を処理します．
    ''' </summary>
    ''' <param name="cmd">送信したコマンド</param>
    ''' 
    ''' <param name="res">応答</param>
    Private Sub tsc_ResponseArrived(cmd As String, res As String)

        If cmd.StartsWith("pcode") Then

            If res.StartsWith("0001") Then
                labProductCode.Content = "0001: TLAN-INS8/8"
            ElseIf res.StartsWith("0002") Then
                labProductCode.Content = "0002: TLAN-INS16I"
            ElseIf res.StartsWith("0003") Then
                labProductCode.Content = "0003: TLAN-INS16O"
            ElseIf res.StartsWith("0004") Then
                labProductCode.Content = "0004: TLAN-08VMA"
            ElseIf res.StartsWith("0005") Then
                labProductCode.Content = "0005: TLAN-08VMD"
            ElseIf res.StartsWith("0006") Then
                labProductCode.Content = "0006: TLAN-RL"
            Else
                labProductCode.Content = "Error or Unknown Device"
            End If

        ElseIf cmd.StartsWith("info") Then
            labInfo.Content = res
        ElseIf cmd.StartsWith("read") Then
            labRead.Content = res

        ElseIf cmd.StartsWith("write") Then
            labWrite.Content = res
        End If

    End Sub

End Class



